/*
 * utilities for the BACI baar archiver
 *    Bill Bynum
 *    May 2000
 */

#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#if !defined(DOS)
#include <unistd.h>
#endif
#include <utime.h>

#include <sys/stat.h>
#include <sys/types.h>

#include "../include/genutil.h"

#include "ar.h"
#include "artables.h"
#include "arutils.h"
#include "arwrite.h"

static void removeSymbols(Archive* ar, Member* tm)
/* 
 *  remove all Symbols from the __.SYMDEF archive member belonging
 *  to the archive Member pointed to by 'tm'
 *
 */
{
   Symbol *ts;
   Symbol *oldts;
   Symbol *ps;

   for (ts = ar->sl.head, ps = 0; ts; ) 
      if (ts->member_offset == tm->offset) {
         if (ps) ps->next = ts->next;
         if (ar->sl.head == ts) ar->sl.head = ts->next;
         if (ar->sl.tail == ts) ar->sl.tail = ps;
         oldts = ts;
         ts = ts->next;
         free(oldts);      /* be tidy */
      }
      else {
         ps = ts; /* no dangling refs! only move ps if ts is not deleted */
         ts = ts->next;
      }
   for (ts = ar->el.head, ps = 0; ts; ) 
      if (ts->member_offset == tm->offset) {
         if (ps) ps->next = ts->next;
         if (ar->el.head == ts) ar->el.head = ts->next;
         if (ar->el.tail == ts) ar->el.tail = ps;
         oldts = ts;
         ts = ts->next;
         free(oldts);      /* be tidy */
      }
      else {
         ps = ts; /* no dangling refs! only move ps if ts is not deleted */
         ts = ts->next;
      }
}  /* removeSymbols */

void deleteMember(Archive* ar, Member* tm, Member* prev, int verbose)
/*
 *  delete the member pointed to by 'tm' from the MemberList of the
 *  archive pointed to by 'ar'
 *  'prev' points to the member of the Member list prior to 
 *  the Member to be deleted
 *  if (verbose) perform the action verbosely
 */
{
      /* first snip out the Member from the Member list */
   if (prev) prev->next = tm->next;
   if (ar->ml.head == tm) ar->ml.head = tm->next;
   if (ar->ml.tail == tm) ar->ml.tail = prev;
      /* now snip out all of the Member's Symbols from the Symbol list */
   if (symdef_present(ar)) removeSymbols(ar,tm);
   if (verbose) printf("d- %s\n",tm->name);
   free(tm);      /* not strictly necessary, but why not be tidy? */
}  /* deleteMember */

void delete_members(Archive* ar, char** files, int num_files, int verbose, 
   int insert_symdef)
/*
 *  delete members named in the list of files 'files' (there
 *  are 'num_files' filenames in the list) from the archive file
 *  described by '*ar'
 *  if (verbose) perform the action verbosely
 *  If 'insert_symdef' and none is there, then add a SYMDEF member 
 *  before deleting.  
 *  Deletion is not allowed unless a SYMDEF is present
 */
{
   Member* tm;
   Member* prev;
   int unchanged = 1;
   read_all_members(ar);
   if (symdef_present(ar)) 
      read_symdef(ar);
   else { /* no symdef */
      if (insert_symdef) { /* add symdef, if asked */
         add_symdef_member(ar,verbose);
         recalc_offsets(ar);
      }
      /* be nice to non SYMDEF-ers! and don't die here */
   }  /* else no symdef */
   while (num_files > 0) {
      if (strcmp(*files, SYMDEF) == 0) {
         nonfatal("Cannot delete %s member\n",SYMDEF);
      }
      else {
         tm = findMember(&ar->ml,&prev,*files);
         if (tm == 0) {
            nonfatal("Member %s not in archive",*files);
         }
         else {
            deleteMember(ar,tm,prev,verbose);
            unchanged = 0;
         }
      }
      num_files--;
      files++;
   }
   if (unchanged) {
      fclose(ar->rf);
      exit(0);
   }
   recalc_offsets(ar);
   write_archive(ar);
}  /* delete_members */


/*
 *  $Log: ardelete.c,v $
 *  Revision 1.12  2003/05/13 21:07:52  bynum
 *  fix bug in removeSymbols
 *
 *  Revision 1.11  2002/08/01 15:23:15  bynum
 *  remove malloc.h include
 *
 *  Revision 1.10  2001/07/20 18:40:54  bynum
 *  add stdlib.h include
 *
 *  Revision 1.9  2001/06/06 20:09:00  bynum
 *  DOS-ize filenames in include stmts, hide unistd.h from DOS compilation
 *
 *  Revision 1.8  2000/06/15 13:49:47  bynum
 *  include genutil.h
 *
 *  Revision 1.7  2000/05/30 15:57:42  bynum
 *  change read_all_members() call
 *
 *  Revision 1.6  2000/05/30 13:48:26  bynum
 *  add code to handle list of external symbols
 *
 *  Revision 1.5  1996/04/25 10:59:03  bynum
 *  change to more graceful behavior when there is no SYMDEF member
 *
 * Revision 1.4  1996/04/23  16:58:30  bynum
 * add address comment, add 'insert_symdef' flag, move code back from
 * 'ar68xx.c' to 'delete_members'
 *
 * Revision 1.3  1996/01/24  17:00:39  bynum
 * change to use of Archive* struct
 *
 * Revision 1.2  1996/01/20  08:25:26  bynum
 * add 'ar_buffers' reference, make several non-public functions static,
 * change 'delete_member' to 'deleteMember', add refs to 'nonfatal'
 *
 * Revision 1.1  1996/01/15  13:39:48  bynum
 * Initial revision
 *
 */
