/*
 *  source code to implement the 'replace' command
 *  for the BACI archiver
 *    Bill Bynum
 *    May 2000
 */
 
#include <errno.h>
#include <time.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "../include/genutil.h"

#include "ar.h"
#include "ardelete.h"
#include "arrdsyms.h"
#include "artables.h"
#include "arutils.h"
#include "arwrite.h"

static void replaceMember(Archive* ar, char* ext_fname, int verbose)
/*
 * if (ar->rf >= 0) (there is an existing archive), then find
 * an existing member of the archive with the given member name
 * and delete it (quietly)
 * Then stat the external file and if it exists, add that member
 * to the archive.
 *
 * if (verbose) perform the action verbosely
 *
 */
{
   char new_name[17];
   int nonverbose = 0;  /* quietly deleteMember */
   Member* tm;
   Member* prev;
   struct stat t_stat;
   FILE* afd = ar->rf;
   if ((afd != NULL)&&(afd != NO_ARCHIVE)) { /* if there's an old archive ... */
      tm = findMember(&ar->ml,&prev,normalize(ext_fname));
      if (tm) deleteMember(ar,tm,prev,nonverbose);
   }
   if (stat(ext_fname,&t_stat) < 0) {
      if (errno == ENOENT) {
         nonfatal("File %s does not exist",ext_fname);
      }
      else {
         fatal("Cannot stat %s",ext_fname);
      }
   }
   else {   /* successful stat */
      tm = newMember();
      if (strlen(ext_fname) > MAX_AR_NAME) {
         strncpy(new_name,ext_fname,MAX_AR_NAME);
         new_name[MAX_AR_NAME-1] = ' ';
         new_name[MAX_AR_NAME] = '\0';
         printf("Member name truncated to '%s'\n",new_name);
         tm->name = save_string(new_name);
      }
      else {   /* not necessary to truncate member name */
         tm->name = save_string(ext_fname);
         tm->mtime = t_stat.st_mtime;
         tm->uid = t_stat.st_uid;
         tm->gid = t_stat.st_gid;
         tm->mode = t_stat.st_mode;
         tm->size = t_stat.st_size;
         tm->ext_fname = save_string(ext_fname);
         appendMember(&ar->ml,tm);
         afd = open_member_file(ext_fname);
         collectMemberSymbols(afd,&ar->sl,&ar->el,tm,verbose);
         fclose(afd);
         if (verbose) printf("r- %s\n",tm->name);
      }  /* else no truncation */
   }  /* else successful stat */
}  /* replaceMember */


void replace_members(Archive* ar,char** files,int num_files,int verbose,
   int insert_symdef)
/*  
 *  replace members named in the list of files 'files' (there
 *  are 'num_files' filenames in the list) from the archive file
 *  described by '*ar'
 *
 *  if (verbose) perform the action verbosely
 *
 *  if (insert_symdef) and there is no SYMDEF member, then create one
 *
 *  if (num_files > 0) and there is no SYMDEF member and !insert_symdef
 *  then abort
 */
{
   read_all_members(ar);
   if (symdef_present(ar)) 
      read_symdef(ar);
   else { /* no symdef */
      if (num_files > 0) {
         /* add symdef, either if asked or if archive is empty */
         if ((insert_symdef)||empty_archive(ar)) { 
            add_symdef_member(ar,verbose);
            recalc_offsets(ar);
            if (!empty_archive(ar))
               collectArchiveSymbols(ar,verbose);
         }
         else { /* no symdef and none called for, tsk, tsk */
            fatal("Cannot replace members in archive %s\n\
Use the 's' option to install a symbol table",ar->fname);
         }  /* files specified */
      }
   }  /* else no symdef */
   if (num_files > 0) {
      while (num_files > 0) {
         replaceMember(ar,*files,verbose);
         num_files--;
         files++;
      }
      recalc_offsets(ar);
      write_archive(ar);      
   }
   else if (ar->rf == NO_ARCHIVE)   /* new_archive, no files to be added */
      write_archive(ar);            /* this will write an empty archive  */
   else  /* no files specified, but archive exists */
      if (verbose) printf("No archive members specified\n");   
}

/*
 *  $Log: arreplac.c,v $
 *  Revision 1.9  2001/06/12 15:10:56  bynum
 *  add verbose parm to collectMemberSymbols and collectArchiveSymbols
 *  calls
 *
 *  Revision 1.8  2001/06/06 20:16:45  bynum
 *  DOS-ize filenames in include stmts
 *
 *  Revision 1.7  2000/05/30 15:49:30  bynum
 *  switch fatal calls to BACI fatal
 *
 * Revision 1.6  1996/06/23  15:56:49  bynum
 * avoid 'collectArchiveSymbols' call when the archive is empty
 *
 * Revision 1.5  1996/04/23  17:05:14  bynum
 * add address comment, move code from 'ar68xx.c' to 'replace_members'
 *
 * Revision 1.4  1996/01/31  10:37:04  bynum
 * add 'create_symdef_member' static func, change 'collectSymbols' call,
 * add 'create_symdef_member' call to 'replace_members' to create a
 * __.SYMDEF member before any other members are added
 *
 * Revision 1.3  1996/01/24  17:05:08  bynum
 * change to use of Archive struct
 *
 * Revision 1.2  1996/01/20  08:33:38  bynum
 * finish entering code for 'replace_members', change 'replace_member' to
 * 'replaceMember'
 *
 * Revision 1.1  1996/01/15  13:39:48  bynum
 * Initial revision
 *
 */
